#include <mainwindow.h>
#include "ui_mainwindow.h"
#include <opengl.h>
#include <variable.h>
#include <structs.h>
#include <functions.h>
#include <iostream>
#include <dxf/read_line.h>
#include <dxf/read_spline.h>
#include <dxf/read_circle.h>
#include <dxf/read_arc.h>
#include <dxf/read_ellipse.h>
#include <dxf/read_dimension.h>
#include <dxf/read_text.h>
#include <cam/contours.h>
#include <cam/dept_sequence.h>
#include <cam/output_sequence.h>

double feedrate;
double cutheight;
double rapidheight;
double pierceheight;
double piercespeed;
double piercedelay;
double spline_resolution; //value >0 && <1
bool spline_G5;
std::string gcode_filename;

#define MyQLineEdit() << QLineEdit() << fixed << qSetRealNumberPrecision(3)

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    timerId = startTimer(200);

    ui->frame_side->hide();
    //example how to connect to buttons:
    //connect(ui->toolButton_new, SIGNAL(clicked()),this, SLOT(on_toolButton_arc_pressed()));
}

MainWindow::~MainWindow()
{
    killTimer(timerId);
    delete ui;
}

void MainWindow::timerEvent(QTimerEvent *)
{
    ui->lineEdit_mx->setText(QString::number(mx,'f',6));
    ui->lineEdit_my->setText(QString::number(my,'f',6));
    ui->lineEdit_message->setText(QString::fromStdString(message));
}

void MainWindow::keyPressEvent(QKeyEvent *event)
{
    int_key=event->key();
    //std::cout<<"key:"<<int_key<<std::endl;
}

void MainWindow::keyReleaseEvent(QKeyEvent *event)
{
    int_key=-1;
}

void MainWindow::on_toolButton_open_pressed()
{
#define DXF_FILES "AC1027 Inkscape dxf R14 files, AC1021 Librecad dxf 2007 files(*.dxf)"
#define GRO_FILES "gro files (*.gro)"

    QFileDialog *dialog = new QFileDialog;
    dialog->resize(500,250);
    QString selectedFilter;

    QString filename_open = QFileDialog::getOpenFileName(
                dialog,
                "Open",
                QDir::currentPath(),
                DXF_FILES ";;" GRO_FILES,
                &selectedFilter,
                QFileDialog::DontUseNativeDialog); //this solves a dxf multiload problem

    if(selectedFilter == GRO_FILES){
        //(read_gro(filename_open));
    }
    if(selectedFilter == DXF_FILES){
        read_line(std::string(filename_open.toStdString()));
        read_spline(std::string(filename_open.toStdString()));
        read_circle(std::string(filename_open.toStdString()));
        read_arc(std::string(filename_open.toStdString()));
        read_ellipse(std::string(filename_open.toStdString()));
        read_dimension(std::string(filename_open.toStdString()));
        read_text(std::string(filename_open.toStdString()));
    }
}

void MainWindow::on_toolButton_line_pressed()
{
    mode="line";
}

void MainWindow::on_toolButton_linestrip_pressed()
{
    mode="linestrip";
}

void MainWindow::on_toolButton_rectangular_pressed()
{
    mode="rectangular";
}

void MainWindow::on_toolButton_circle_pressed()
{
    mode="circle";
}

void MainWindow::on_toolButton_ellipse_pressed()
{
    mode="ellipse";
}

void MainWindow::on_toolButton_arc_pressed()
{
    mode="arc";
}

void MainWindow::on_toolButton_spline_pressed()
{
    mode="spline";
}

void MainWindow::on_toolButton_bezier_pressed()
{
    mode="bezier";
}

void MainWindow::on_toolButton_polygon_pressed()
{
    mode="polygon";
}

void MainWindow::on_toolButton_new_pressed()
{
    objects.clear();

}

void MainWindow::on_toolButton_cam_pressed()
{
    mode="";
    ui->frame_side->show();
    ui->progressBar->setValue(0);
}

void MainWindow::on_pushButton_process_pressed()
{
    mode="";
    feedrate=ui->lineEdit_cutspeed->text().toDouble();
    cutheight=ui->lineEdit_cutheight->text().toDouble();
    rapidheight=ui->lineEdit_travelheight->text().toDouble();
    pierceheight=ui->lineEdit_pierceheight->text().toDouble();
    piercespeed=ui->lineEdit_piercespeed->text().toDouble();
    piercedelay=ui->lineEdit_piercedelay->text().toDouble();
    spline_resolution=ui->lineEdit_spline_resolution->text().toDouble(); //value >0 && <1
    spline_G5=false;
    gcode_filename=ui->lineEdit_filename->text().toStdString();

    ui->progressBar->setValue(10);
    ui->textEdit_gode->clear();

    find_contours();
    ui->progressBar->setValue(50);
    keep_parts_together();
    ui->progressBar->setValue(60);
    area();
    ui->progressBar->setValue(80);
    map_cw_ccw();
    ui->progressBar->setValue(90);
    process_gcode();
    ui->progressBar->setValue(100);

    //display gcode in textbox
    QString string = QString::fromStdString(gcode_filename);
    QFile file(string);
    QString line;
    if (file.open(QIODevice::ReadOnly | QIODevice::Text)){
        QTextStream stream(&file);
        while (!stream.atEnd()){

            line.append(stream.readLine()+"\n");
        }
        //testLabel->setText(line);
        ui->textEdit_gode->setText(line);
    }
    file.close();
}

void MainWindow::on_pushButton_close_pressed()
{
    ui->frame_side->close();
    ui->progressBar->setValue(0);
}





























